from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from inbox.models import Inbox


class HomeViewTest(TestCase):
    def test_view_url_exists_at_desired_location(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 302)

    def test_view_url_accessible_by_name(self):
        response = self.client.get(reverse('home'))
        self.assertEqual(response.status_code, 302)


class SetupClass(TestCase):
    def setUp(self):
        self.owner = User.objects.create_user(username='owner', password='test')
        self.guess = User.objects.create_user(username='guess', password='test')
        self.inbox = Inbox.objects.create(owner=self.owner, guess=self.guess)
        self.client.login(username='owner', password='test')


class InboxTest(SetupClass):
    def test_view_url_exists_at_desired_location(self):
        response = self.client.get('/inbox/1/')
        self.assertEqual(response.status_code, 200)

    def test_view_url_accessible_by_name(self):
        response = self.client.get(reverse('view-inbox', kwargs={'pk': '1'}))
        self.assertEqual(response.status_code, 200)


class NewInboxTest(SetupClass):
    def test_view_url_exists_at_desired_location(self):
        response = self.client.get('/new-inbox/')
        self.assertEqual(response.status_code, 200)

    def test_view_url_accessible_by_name(self):
        response = self.client.get(reverse('new-inbox'))
        self.assertEqual(response.status_code, 200)

    def test_new_inbox_view(self):
        pass


class NewMessageTest(SetupClass):
    def test_view_url_exists_at_desired_location(self):
        response = self.client.post('/new-message/1/')
        self.assertEqual(response.status_code, 302)

    def test_view_wrong_method(self):
        response = self.client.get('/new-message/1/')
        self.assertEqual(response.status_code, 403)



from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import redirect, render

from .models import Inbox
from .forms import InboxForm, MessageForm


@login_required
def home(request):
    inboxes = Inbox.objects.filter(Q(owner=request.user) | Q(guess=request.user))
    inbox = inboxes.order_by('-updated_at').first()
    if not inbox:
        return render(request, 'inbox/inboxes.html', {})
    else:
        return redirect('view-inbox', pk=inbox.pk)


@login_required
def view_inbox(request, pk):
    inboxes = Inbox.objects.filter(Q(owner=request.user) |
                                   Q(guess=request.user)).order_by('-updated_at')
    inbox = Inbox.objects.get(pk=pk)
    if (request.user == inbox.owner) or (request.user == inbox.guess):
        messages = inbox.messages.all()
        # Mark with read=True all the messages from the other user
        for message in messages.filter(Q(read=False) & ~Q(user=request.user)):
            message.read = True
            message.save()
        form = MessageForm()
        return render(request, 'inbox/inbox-view.html', {'inbox': inbox, 'inboxes': inboxes,
                                                         'form': form, 'messages': messages})
    else:
        return HttpResponse(status=403)


@login_required
def new_inbox(request):
    if request.method == 'POST':
        form = InboxForm(request.user, request.POST)
        if form.is_valid():
            inbox = form.save(commit=False)
            inbox.owner = request.user
            inbox.save()
            return redirect('home')
    else:
        form = InboxForm(request.user)
    return render(request, 'inbox/new-inbox.html', {'form': form})


@login_required
def new_message(request, pk):
    inbox = Inbox.objects.get(pk=pk)
    if request.method == 'POST':
        form = MessageForm(request.POST)
        if form.is_valid():
            message = form.save(commit=False)
            message.user = request.user
            message.inbox = inbox
            message.save()

        return redirect('view-inbox', pk=inbox.pk)
    else:
        return HttpResponse(status=403)



from django import template
from django.db.models import Q

from inbox.models import Inbox

register = template.Library()


@register.simple_tag
def get_unread_messages(inbox, user):
    inbox = Inbox.objects.get(pk=inbox.pk)
    unread_messages = inbox.messages.filter(Q(read=False) & ~Q(user=user))
    return unread_messages.count()



from django.urls import path

from . import views

urlpatterns = [
    path('', views.home, name='home'),
    path('view/<int:pk>/', views.view_inbox, name='view-inbox'),
    path('new/', views.new_inbox, name='new-inbox'),
    path('new-message/<int:pk>/', views.new_message, name='new-message')
]



from django.contrib.auth.models import User
from django.test import TestCase

from inbox.models import Inbox, Message


class SetupClass(TestCase):
    def setUp(self):
        self.owner = User.objects.create_user(username='owner', password='test')
        self.guess = User.objects.create_user(username='guess', password='test')
        self.inbox = Inbox.objects.create(owner=self.owner, guess=self.guess)


class InboxTest(SetupClass):
    def test_inbox_creation(self):
        self.assertTrue(isinstance(self.inbox, Inbox))
        self.assertEqual(str(self.inbox), 'owner - guess')
        self.assertEqual(self.inbox.owner, self.owner)
        self.assertEqual(self.inbox.guess, self.guess)


class MessageTest(SetupClass):
    def test_message_creation(self):
        message = Message.objects.create(inbox=self.inbox, user=self.owner, content='Hello world')
        self.assertTrue(isinstance(message, Message))
        self.assertEqual(message.user, self.owner)
        self.assertEqual(message.content, 'Hello world')
        self.assertFalse(message.read)



from django.db import models
from django.contrib.auth.models import User


class TimestampMixin(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Inbox(TimestampMixin):
    owner = models.ForeignKey(User, on_delete=models.CASCADE, related_name='owner_inboxes')
    guess = models.ForeignKey(User, on_delete=models.CASCADE, related_name='guess_inboxes')

    def __str__(self):
        return self.owner.username + ' - ' + self.guess.username

    class Meta:
        verbose_name_plural = 'inboxes'


class Message(TimestampMixin):
    inbox = models.ForeignKey(Inbox, on_delete=models.CASCADE, related_name='messages')
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='messages')
    content = models.TextField()
    read = models.BooleanField(default=False)

    def __str__(self):
        return self.content[:25]

    def save(self, *args, **kwargs):
        self.inbox.save()
        super(Message, self).save(*args, **kwargs)

    class Meta:
        ordering = ['created_at']



from django.contrib.auth.models import User
from django.forms import ModelForm

from .models import Inbox, Message


class InboxForm(ModelForm):
    class Meta:
        model = Inbox
        fields = ['guess']

    def __init__(self, user, *args, **kwargs):
        super(InboxForm, self).__init__(*args, **kwargs)
        self.fields['guess'].queryset = User.objects.exclude(pk=user.pk)


class MessageForm(ModelForm):
    class Meta:
        model = Message
        fields = ['content']



# Generated by Django 2.1.3 on 2018-11-03 11:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Inbox',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('guess', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='guess_inboxes', to=settings.AUTH_USER_MODEL)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='owner_inboxes', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'inboxes',
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('content', models.TextField()),
                ('read', models.BooleanField(default=False)),
                ('inbox', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='inbox.Inbox')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['created_at'],
            },
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as rf:
    README = rf.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-inbox',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description=['A not live messaging system that simulates an email inbox'],
    long_description=README,
    url='https://gitlab.com/polrodoreda/django-inbox',
    author='Pol Rodoreda Valeri',
    author_email='pol_rodoreda@hotmail.com',
    install_requires=[
        'Django>=2.0'
    ]
)



from django.contrib.auth.models import User
from django.test import TestCase

from inbox.models import Inbox
from inbox.forms import InboxForm, MessageForm


class SetupClass(TestCase):
    def setUp(self):
        self.owner = User.objects.create_user(username='owner', password='test')
        self.guess = User.objects.create_user(username='guess', password='test')
        self.inbox = Inbox.objects.create(owner=self.owner, guess=self.guess)
        self.client.login(username='owner', password='test')


class InboxFormTest(SetupClass):
    def test_inbox_form_valid(self):
        data = {'guess': self.guess.pk}
        form = InboxForm(user=self.owner, data=data)
        self.assertTrue(form.is_valid())


class MessageFormTest(SetupClass):
    def test_message_form_valid(self):
        data = {'content': 'Hello'}
        form = MessageForm(data=data)
        self.assertTrue(form.is_valid())



from django.contrib import admin

from .models import Inbox, Message


@admin.register(Inbox)
class InboxAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_at')
    readonly_fields = ('created_at', 'updated_at')


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    list_display = ('pk', 'created_at')
    readonly_fields = ('created_at', 'updated_at')
